"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@ts-ioc/core");
const DIModuleValidate_1 = require("./DIModuleValidate");
const DIModule_1 = require("../decorators/DIModule");
const utils_1 = require("../utils");
const InjectedModule_1 = require("./InjectedModule");
/**
 * DIModule injector token.
 */
exports.DIModuleInjectorToken = new core_1.InjectModuleInjectorToken(DIModule_1.DIModule.toString());
/**
 * DIModule injector.
 *
 * @export
 * @class DIModuleInjector
 * @extends {ModuleInjector}
 */
let DIModuleInjector = class DIModuleInjector extends core_1.ModuleInjector {
    constructor(validate) {
        super(validate);
    }
    setup(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.importModule(container, type);
        });
    }
    syncSetup(container, type) {
        throw new Error('DIModule can not sync setup.');
    }
    valid(container, type) {
        if (!this.validate) {
            return true;
        }
        return this.validate.valid(type);
    }
    getMetaAccessor(container, token) {
        return container.getService(core_1.MetaAccessorToken, token);
    }
    importModule(container, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let pools = container.get(utils_1.ContainerPoolToken);
            let newContainer = pools.create(type, container);
            newContainer.register(type);
            let decorator = this.validate.getDecorator();
            let accor = this.getMetaAccessor(newContainer, decorator);
            let metaConfig = accor.getMetadata(type, newContainer, undefined, decorator ? dec => dec === decorator : undefined);
            yield this.registerConfgureDepds(newContainer, metaConfig, type);
            let exps = yield this.getConfigExports(newContainer, metaConfig);
            let injMd = new InjectedModule_1.InjectedModule(metaConfig.token || type, metaConfig, newContainer, type, exps);
            container.bindProvider(new core_1.InjectReference(core_1.ProviderMap, type), exps);
            container.bindProvider(new InjectedModule_1.InjectedModuleToken(type), injMd);
            yield this.importConfigExports(container, newContainer, injMd);
            return injMd;
        });
    }
    registerConfgureDepds(container, config, type) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (core_1.isArray(config.imports) && config.imports.length) {
                yield container.loadModule(...config.imports);
            }
        });
    }
    getConfigExports(container, config) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let parser = container.getProviderParser();
            let map = parser.parse(...config.providers || []);
            // bind module providers
            container.bindProviders(map);
            let builder = container.getBuilder();
            let exptypes = [].concat(...builder.loader.getTypes(config.exports || []));
            exptypes.forEach(ty => {
                let classPd = container.resolveValue(new core_1.InjectClassProvidesToken(ty));
                map.add(ty, (...pds) => container.resolve(ty, core_1.ResoveWay.nodes, ...pds));
                if (classPd && core_1.isArray(classPd.provides) && classPd.provides.length) {
                    classPd.provides.forEach(p => {
                        if (!map.has(p)) {
                            map.add(p, (...pds) => container.resolve(p, core_1.ResoveWay.nodes, ...pds));
                        }
                    });
                }
            });
            return map;
        });
    }
    importConfigExports(container, providerContainer, injMd) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (container === providerContainer) {
                return container;
            }
            if (injMd) {
                let chain = container.getResolvers();
                chain.next(injMd);
                //     if (injMd.exports && injMd.exports.size) {
                //         // injMd.exports.forEach(exp => {
                //         //     if (isClass(exp) && hasOwnClassMetadata(IocExt, exp)) {
                //         //         root.register(exp);
                //         //     }
                //         // });
                //         providerContainer.getResolvers().toArray(ResoveWay.traverse).forEach((r: IExports) => {
                //             if (r.type && injMd.exports.has(r.type)) {
                //                 console.log(injMd.type, r.type);
                //                 chain.next(r);
                //                 r.exports.forEach(exp => {
                //                     if (isClass(exp) && hasOwnClassMetadata(IocExt, exp)) {
                //                         container.register(exp);
                //                     }
                //                 });
                //             }
                //         });
                //     }
            }
            return container;
        });
    }
    static getClassAnnations() {
        return { "name": "DIModuleInjector", "params": { "constructor": ["validate"], "setup": ["container", "type"], "syncSetup": ["container", "type"], "valid": ["container", "type"], "getMetaAccessor": ["container", "token"], "importModule": ["container", "type"], "registerConfgureDepds": ["container", "config", "type"], "getConfigExports": ["container", "config"], "importConfigExports": ["container", "providerContainer", "injMd"] } };
    }
};
DIModuleInjector = tslib_1.__decorate([
    core_1.Singleton(exports.DIModuleInjectorToken),
    tslib_1.__param(0, core_1.Inject(DIModuleValidate_1.DIModuleValidateToken)),
    tslib_1.__metadata("design:paramtypes", [Object])
], DIModuleInjector);
exports.DIModuleInjector = DIModuleInjector;

//# sourceMappingURL=../sourcemaps/modules/DIModuleInjector.js.map
